#include "fresh.h"

#include <Arduino.h>
#include <time.h>

/**
 * @brief The allowed discrepancy between the time of a received message
 * and the device time (in seconds)
 * 
 * A stricter (lower) value better prevents against replay attacks,
 * but may lead to issues when dealing with slow networks and other
 * routing delays.
 */
uint32_t allowedOffset = 60;

void setMessageTimeAllowedOffset(uint32_t offset) {
    allowedOffset = offset;
}

void configureNTP(int32_t offsetToGMT, int32_t offsetDaylightSavings, const char* serverUrl) {
    configTime(offsetToGMT, offsetDaylightSavings, serverUrl);
}

void printLocalTime() {
    struct tm timeinfo;
    if (getLocalTime(&timeinfo)) {
        Serial.println(&timeinfo, "[INFO] Time is %A, %d. %B %Y %H:%M:%S");
    } else {
        Serial.println("[WARN] No local time available");
    }
}

uint32_t getEpochTime() {
    time_t now;
    struct tm timeinfo;
    if (!getLocalTime(&timeinfo)) {
        Serial.println("[WARN] Failed to obtain local time");
        return(0);
    }
    time(&now);
    return now;
}

bool isMessageTimeAcceptable(uint32_t t) {
    uint32_t localTime = getEpochTime();
    if (localTime == 0) {
        Serial.println("No epoch time available");
        return false;
    }
    if (t > localTime + allowedOffset) {
        return false;
    }
    if (t < localTime - allowedOffset) {
        return false;
    }
    return true;
}